@use "@ag-grid-community/styles" as ag;

@include ag.grid-styles((
    themes: (
      balham:(),
      balham-dark:(),
      alpine:(),
      alpine-dark:(),
      material:(),
      streamlit: (
        extend-theme: alpine,
        background-color: #ffffff,
        odd-row-background-color: #ffffff,
        foreground-color: #31333F,
        alpine-active-color: #ff4b4b,
        grid-size: 4px,
        header-background-color: #f8f9fb,
        borders: solid 0.5px,
        border-color: #eaeaeb,
        cell-horizontal-border: solid #eaeaeb,
        header-foreground-color: #7f838a,
        font-family: "Source Sans Pro",
        font-size: 9.5pt
      ),
      streamlit-dark: (
        extend-theme: alpine-dark,
        background-color: #0d1116,
        odd-row-background-color: #0d1116,
        foreground-color: #ffffff,
        alpine-active-color: #ff4b4b,
        grid-size: 4px,
        header-background-color: #1a1c24,
        borders: solid 0.5px,
        border-color: #303239,
        cell-horizontal-border: solid #303239,
        header-foreground-color: #7f838a,
        font-family: "Source Sans Pro",
        font-size: 9.5pt

      ),
    ),
));

@import "@fontsource/source-sans-pro";